#Index

**Functions**

* [waitFor(args*)](#waitFor)
* [config(envelope)](#config)
* [load(key, val, [options])](#load)
* [load(key, val)](#load)
* [register(key, val)](#register)
* [isEvent(key)](#isEvent)
* [isRemote(val)](#isRemote)
* [waitForRemote(key)](#waitForRemote)
* [fetch(options, dfd, Depth)](#fetch)
* [preload(context)](#preload)
* [status(key)](#status)
* [watchForDependencyTimeout(dfd, key)](#watchForDependencyTimeout)
* [getModuleName(identifier)](#getModuleName)
* [unload(key)](#unload)
 
<a name="waitFor"></a>
#waitFor(args*)
Request a module. If it is not already in the system, create
a deferred for it, to be resolved when it actually loads.

**Params**

- args* `String` | `Array` - One or more strings, or an array of
strings, representing module keys  

**Returns**: `Promise` - A promise that will be resolved with the requested modules in passed order  
<a name="config"></a>
#config(envelope)
Provides a method interface for configurable options to maintain some semblance of privacy

**Params**

- envelope `Object`  

**Returns**: `void`  
<a name="load"></a>
#load(key, val, [options])
First point of entry for modules to register with the system.

### Examples

    // any type can be loaded
    waiter.load("SomeKey", {data: [1,2,3,4]});
    
    // if it's a url, we'll try to load it
    waiter.load('myData', 'http://my.dynamic-remote.com/');

**Params**

- key `String` - The module name/key.  
- val `Varied` - A module that should be used to resolve those waiting for `key` or a url that should be fetched.  
- \[options\] `Object` - Optional specifying handling of module.  
  - \[exports\] `String` - Specifies where the module is in the namespace e.g. `'_'` for underscore  
  - \[dataType\] `String` - dataType to be passed to jQuery ajax call  

<a name="load"></a>
#load(key, val)
First point of entry for modules to register with the system.

**Params**

- key `String` - The module name/key.  
- val `Varied` - The module value.  

<a name="register"></a>
#register(key, val)
Register with the dependency management system, resolving any existing
deferreds with the same key (because a module already waited for it
before it could register), or creating a resolved defered otherwise.

**Params**

- key `String` - The module name/key.  
- val `Varied` - The module value.  

<a name="isEvent"></a>
#isEvent(key)
Checks a key to see if it fits our event syntax

**Params**

- key `String`  

**Returns**: `Boolean`  
<a name="isRemote"></a>
#isRemote(val)
Checks a val to see if it is a url

**Params**

- val `String`  

**Returns**: `Boolean`  
<a name="waitForRemote"></a>
#waitForRemote(key)
Special waitFor method that deals with remote modules.

**Params**

- key `String` - The module key that should be prepended with
'Remote:' to access the envelope.  

<a name="fetch"></a>
#fetch(options, dfd, Depth)
Recursive method that retries an AJAX request with gaps specified in settings.retries. The passed
Deferred is resolved when the request is completed. Calls settings.errorHandler on a failure.

**Params**

- options `Object` - Used to population $.ajax request  
- dfd `Deferred` - To be resolved on completion of request  
- Depth `Number` - counter for recursion  

**Returns**: `void`  
<a name="preload"></a>
#preload(context)
Load dependencies, specified as an array of dependency names, for a
given `context` object onto that object.

**Params**

- context `Object` - The object that will have its dependencies
preloaded.  

<a name="status"></a>
#status(key)
Helper method to inspect the state of a module that may or may not be loading or already loaded. Returns undefined
if the passed key is not registered.

**Params**

- key `String` - Name of desired module  

**Returns**: `String` | `undefined` - Element of the set ["resolved", "pending", "rejected", undefined]  
<a name="watchForDependencyTimeout"></a>
#watchForDependencyTimeout(dfd, key)
Uses settings.timeout to determine how long to wait before calling settings.errorHandler

**Params**

- dfd `Promise`  
- key `String`  

**Returns**: `void`  
<a name="getModuleName"></a>
#getModuleName(identifier)
Convert a colon-separated namespace `identifier` string to an object
name.

**Params**

- identifier `String` - The colon separated string to convert.  

<a name="unload"></a>
#unload(key)
Remove a module from the system.

**Params**

- key `String` - The key for the module to be removed.  

